package irai; 
$irai::VERSION = '0.003';
use strict;
use warnings;

=head1 NAME

irai - enables L<Future> debugging UI for script

=head1 VERSION

version 0.003

=head1 DESCRIPTION

See L<App::mirai>.

=cut

# Actual work is done by loading this module
use App::mirai::Future;

1;

__END__

=head1 AUTHOR

Tom Molesworth <cpan@perlsite.co.uk>

=head1 LICENSE

Copyright Tom Molesworth 2014-2015. Licensed under the same terms as Perl itself.
